﻿function textoComDestaque(item, valor) {
    return $("<p />").append($("<strong />").text(item + ":"), " " + valor);
}

function atualizaCalendario() {
    var c = $("#calendar");
    c.fullCalendar("refetchResources");
    c.fullCalendar("refetchEvents");
}

// Retorna null se v1 == v2, senão, retorna v1
function nullIf(v1, v2) {
    return (v1 == v2) ? null : v1;
}

// Retorna null caso o valor informado for uma string vazia, senão, retorna o próprio valor
function trocaStringVaziaPorNull(valor) {
    return nullIf(valor, "");
}

function montaVisaoDetalhes(dados) {
    // Adiciona um item de descrição nos detalhes, mas apenas caso ele possua valor
    var adicionaSeExiste = function (descricao, valor) {
        if (valor)
            conteudo.append(textoComDestaque(descricao, valor));
    }

    var conteudo = $("<div />");
    conteudo.append($("<h2 />").text(dados.title));
    // Não adiciona o título dos detalhes da atividade caso ele seja igual ao título do evento
    if (dados.title != dados.eventInformation.tituloEvento)
        conteudo.append($("<h3 />").text(dados.eventInformation.tituloEvento));
    conteudo.append(textoComDestaque("Início", (dados.descriptionStart ? moment(dados.descriptionStart) : dados.start).format("lll")));
    if (dados.end)
        conteudo.append(textoComDestaque("Fim", (dados.descriptionEnd ? moment(dados.descriptionEnd) : dados.end).format("lll")));
    if (dados.roomInformation) {
        if (dados.roomInformation.tipoSala)
            conteudo.append(textoComDestaque("Tipo de sala", dados.roomInformation.tipoSala.nome));
        conteudo.append(textoComDestaque("Sala", dados.roomInformation.sala));
    }

    switch (dados.eventInformation.tipo) {
        case "plano":
            adicionaSeExiste("Disciplina", dados.eventInformation.disciplina);
            adicionaSeExiste("Turma", dados.eventInformation.turma);
            adicionaSeExiste("Professor", dados.eventInformation.professor);
            adicionaSeExiste("Quantidade de alunos", dados.eventInformation.qtdeAlunos);
            break;
        case "agendada":
        case "tfc":
            adicionaSeExiste("Solicitado por", dados.eventInformation.responsavel);
            adicionaSeExiste("Solicitado em", moment(dados.eventInformation.dataReserva).format("lll"));
            adicionaSeExiste("Complemento", dados.eventInformation.complemento);
            break;
        case "ofertada":
            adicionaSeExiste("Solicitado por", dados.eventInformation.responsavel);
            adicionaSeExiste("Solicitado em", moment(dados.eventInformation.dataReserva).format("lll"));
            break;
        case "recurso":
            adicionaSeExiste("Atividade", dados.eventInformation.nomeAtividade);
            adicionaSeExiste("Solicitado por", dados.eventInformation.responsavel);
            adicionaSeExiste("Solicitado em", moment(dados.eventInformation.dataReserva).format("lll"));
            adicionaSeExiste("Complemento", dados.eventInformation.complemento);
            break;
        case "livre":
            break;
    }

    if (dados.eventInformation.recursosAdicionais) {
        conteudo.append($("<p />").append($("<strong />").text("Recursos adicionais:")));
        var lista = $("<ul />");
        for (var i = 0; i < dados.eventInformation.recursosAdicionais.length; i++)
            lista.append($("<li />").text(dados.eventInformation.recursosAdicionais[i]));
        conteudo.append(lista);
    }

    return conteudo;
};

// Remove as opções que foram adicionadas à um select.
// Todas as opções serão removidas, com exceção daquela em que o valor é "*" (ou seja, o "todos")
function removeOpcoesDinamicas(selector) {
    $(selector + " option[value!='*']").each(function (idx, value) { value.parentElement.removeChild(value); });
}

// Retorna o valor do parâmetro "name" que foi passado por querystring para a página atual
function getParameterByName(name) {
    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)");
    var results = regex.exec(location.search);
    return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
}

// Preenche os dropdowns de filtro com os valores dos parâmetros passados por querystring
// Esta função é chamada uma vez, no carregamento da página
function preencheCamposQueryString() {

    // Busca, na querystring, um parâmetro com o nome informado e aplica o seu valor à um elemento de mesmo
    // ID na página. Retorna "True" caso ele conseguiu fazer este processo (ou seja, encontrou o parâmetro
    // informado e encontrou um elemento para aplicar este valor), ou false caso não conseguiu (não encontrou
    // o parâmetro, o parâmetro estava em branco ou o elemento não foi encontrado).
    // Observação: após aplicar o valor, os eventos "onchange" existentes no controle afetado serão disparados,
    // assim como ocorreria caso a alteração tivesse sido feita pela interface
    var preenche = function (campo) {
        var qs = getParameterByName(campo);
        if (qs != "") {
            var el = $("#" + campo);
            if (el) {
                el.val(qs);
                el.change();
                return true;
            }
        }
        return false;
    }

    preenche("ddlTurno");
    preenche("ddlTipoSala");
    preenche("ddlCampus");
    preenche("ddlPredio");
    preenche("ddlBloco");
    preenche("ddlSala");
    preenche("ddlTipoRecurso");
    preenche("ddlRecurso");
}

// Retorna o valor armazenado no campo informado.
// Caso ele estiver com a opção "Todos" selecioanda (*), retorna ""
function getOptionValueOrEmpty(selector) {
    var obj = $(selector).val();
    if (obj == "*")
        return "";
    return obj;
}

// Inicializa o calendário de reservas. Caso ele já exista, exclui ele e cria denovo
// opcoesCalendario é um objeto que será passado ao criar o objeto FullCalendar. Algumas
// opções já são definidas por padrão, mas pode-se, a partir deste objeto, alterar os valores
// dessas opções padrão
function internalInicializaCalendario(opcoesCalendario) {
    // Busca o início e fim do intervalo a ser mostrado
    var inicio = "06:00";
    var fim = "23:00";
    var regex = /([0-9]{2}:[0-9]{2}) - ([0-9]{2}:[0-9]{2})/g
    var opt = $("#ddlTurno > option:selected").text();
    var match = regex.exec(opt);
    if (match != null) {
        inicio = match[1];
        fim = match[2];
    }
    inicio = moment.duration(inicio).subtract(moment.duration("01:00"));
    fim = moment.duration(fim).add(moment.duration("01:00"));

    // Inicialização do calendário
    var c = $("#calendar");
    if (c == null)
        throw 'O elemento onde o calendário deverá ser posicionado não foi encontrado.';

    // Pega os valores em uso no momento (caso o calendário será regarado) ou define os valores padrão
    // para a criação do objeto
    var theView = 'timelineDay';
    var theDate = moment().format('YYYY-MM-DD');

    if (c.children().length > 0) {
        theView = c.fullCalendar('getView').name;
        theDate = c.fullCalendar('getDate').format('YYYY-MM-DD');
        c.fullCalendar('destroy');
    }

    // Inicializa o calendário
    var opcoesPadrao = {
        header: {
            left: 'prev,next today',
            center: 'title',
            right: 'month,agendaWeek,timelineDay'
        },
        editable: false,
        eventLimit: true,
        defaultView: theView,
        defaultDate: theDate,
        navLinks: true,
        refetchResourcesOnNavigate: true,
        height: window.innerHeight - (window.innerHeight > 150 ? 150 : 0),
        lazyFetching: false,
        businessHours: {
            start: "07:00",
            end: "23:00",
            dow: [1, 2, 3, 4, 5, 6]
        },
        minTime: inicio,
        maxTime: fim,
        allDaySlot: false,
        slotEventOverlap: false,
        loading: function (state, view) {
            if (state)
                $("#carregando").show();
            else
                $("#carregando").hide();
        },
        schedulerLicenseKey: 'GPL-My-Project-Is-Open-Source'
    };

    c.fullCalendar($.extend({}, opcoesPadrao, opcoesCalendario));
}
